%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     This code defines the different parameters of the scenario to study: 
%   position of the sensors, sources to estimate and generate the
%   observations
%
%   This code will generate NbMcRunsMain .mat file that will be saved
%   in the SavedData/ directory for which:
%      - the sources' position and power is drawn from the prior
%      - the observations is generated according to the parameters
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Matlab Codes associated to the paper
%       ``A Bayesian Perspective on Multiple Source
%        Localization in Wireless Sensor Network'' 
% by T.L.T Nguyen, F. Septier, H. Rajaona, G.W. Peters, I. Nevat,
%       Y. Delignon
%  
%       Date: Sep. 2015
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all;
clear all;

NbMcRunsMain=10; % Number of scenarios to generate

Display=1; % 1- Depict the generated scenario on the fly | 0 - No Figures

addpath('Distributions/')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parameters related to the SENSORS and the DOMAIN
SignalDecay=2; % Signal Decay or Path Loss Exponent
Sensors.SignalDecay=SignalDecay;
Sensors.Nb=10^2; % Number of sensors in the domain
Sensors.Domain=[0 100]; % Size of the domain
Sensors.NoiseVariance=1; % Variance of the observation noise
Sensors.NbMeasurements=1; % Number of measurements Per Sensor


% Location of the sensors - Actually equally spaced (i.e. grid) in the domain
[Xsensors,YSensors]=meshgrid(linspace(Sensors.Domain(1),Sensors.Domain(2),sqrt(Sensors.Nb)));
Sensors.Position=reshape(Xsensors,1,[]);
Sensors.Position=[Sensors.Position;reshape(YSensors,1,[])];


% Parameters of the quantization step
Sensors.NbLevel=8;  % Number of quantization Levels
Sensors.Threshold=linspace(0,sqrt(500),Sensors.NbLevel-1); % Threshold - Actually equally spaced


% Channel Transmission between sensors and Fusion Center
% Actually its a Transition Probabiliyt Matrix as defined in the paper
ProbaCorrectTrans=0.9; % Probability of correclty transmitting the true quant. level
Others=exp(-[0:Sensors.NbLevel-2]/1);
Sensors.TransMatrix=[0 Others];
for n=2:Sensors.NbLevel  
    NewLine=[Sensors.TransMatrix(1,n) Sensors.TransMatrix(n-1,1:end-1)];
    Sensors.TransMatrix=[Sensors.TransMatrix;NewLine];
end
Sensors.TransMatrix=(1-ProbaCorrectTrans)*Sensors.TransMatrix./repmat(sum(Sensors.TransMatrix,2),1,Sensors.NbLevel);
Sensors.TransMatrix=Sensors.TransMatrix+ProbaCorrectTrans*eye(Sensors.NbLevel);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parameters related to the SOURCES and a priori knowledge
Target.Nb=4; % True Number of sources to infer
Target.Dim=2*Target.Nb;
Target.d0=1; % Reference Distance

% Parameters of the prior for the transmitted power of each source
Target.PowerShape=50;   % Shape Parameter for IG prior  - Transmitted Power
Target.PowerScale=250000;% Scale Parameter for IG prior - Transmitted Power

% Parameters of the prior for the position of each source
Target.PriorMean=0.5*(Sensors.Domain(2)-Sensors.Domain(1))*ones(Target.Dim,1);
Target.PriorCov=0.9*((0.5*(Sensors.Domain(2)-Sensors.Domain(1))/2.807033768344)^2)*eye(Target.Dim); %2.807033768344 used in order to cover 99% of the domain



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  The following code generate the NbMcRunsMain scenarios that will
%       be saved in NbMcRunsMain individual .mat files
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for n=1:NbMcRunsMain 

    
	Target.Position=[];
	Target.Power=[];    
    
    % Sampling of the position and power from the prior for each source
    Target.Position=[Target.Position;Target.PriorMean+sqrtm(Target.PriorCov)*randn(Target.Dim,1)];% Sample at each time new target position
    Target.Power=[Target.Power;1./gamrnd(Target.PowerShape,1/Target.PowerScale,Target.Nb,1)];
    
    
    if Display
    
        figure(512);clf;
        plot(Sensors.Position(1,:),Sensors.Position(2,:),'o')
        for nbTarget=1:Target.Nb
            IndexTarget=(nbTarget-1)*Target.Dim/Target.Nb+1:(nbTarget)*Target.Dim/Target.Nb;
            
            hold on;plot(Target.Position(IndexTarget(1)),Target.Position(IndexTarget(2)),'sr')
        end
        axis(repmat(Sensors.Domain,1,2))
        drawnow;
        
    end

    
    % Generation of the signal amplitude that will be received at all the
    % different sensors
    A=zeros(Target.Nb,Sensors.Nb);
    Distance=zeros(Target.Nb,Sensors.Nb);  
    for nbTarget=1:Target.Nb
        IndexTarget=(nbTarget-1)*Target.Dim/Target.Nb+1:(nbTarget)*Target.Dim/Target.Nb;
        Distance(nbTarget,:)=sqrt(sum(abs(repmat(Target.Position(IndexTarget,1),1,Sensors.Nb)-Sensors.Position).^2,1));
        A(nbTarget,:)=sqrt(Target.Power(nbTarget)*(Target.d0./Distance(nbTarget,:)).^SignalDecay);  
    end
    A=sum(A,1);
    
    % Add some random Gaussian Noise from the measurement
    Sensors.Observations=(A)+sqrt(Sensors.NoiseVariance)*randn(Sensors.NbMeasurements,Sensors.Nb);

    % Quantization Step
    Sensors.QuantObs=sum(repmat(Sensors.Threshold',1,Sensors.Nb)<repmat(Sensors.Observations(1,:),Sensors.NbLevel-1,1),1);

    % Received Levels received at the Fusion Center (FC) that take into account
    % error in the transmission between sensors and FC.
    for numSensors=1:Sensors.Nb
        Sensors.ReceivedSymbols(1,numSensors)=randsample(Sensors.NbLevel,1,true,Sensors.TransMatrix(Sensors.QuantObs(numSensors)+1,:))-1;
    end
    

    % Save the simulated scenario into a .mat file
    FileToSave=['save SavedData/TargetScenario_NbSensors=',num2str(Sensors.Nb),'_QuantNbLevel=',num2str(Sensors.NbLevel),'_NoiseVar=',num2str(Sensors.NoiseVariance),'_NumMcRun=',num2str(n),'.mat'];
    eval(FileToSave)
    
    
    
end

