%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     This code performs the Importance Sampler
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Matlab Codes associated to the paper
%       ``A Bayesian Perspective on Multiple Source
%        Localization in Wireless Sensor Network'' 
% by T.L.T Nguyen, F. Septier, H. Rajaona, G.W. Peters, I. Nevat,
%       Y. Delignon
%  
%       Date: Sep. 2015
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [SMCSampler]=ImportanceSampler(SMCSampler,Target,CurrentNbMCMCMoves,CurrentNbBlockMWG,Sensors,PercentageCESS);


SMCSampler.NbIter=1; % Case of importance Sampling


LogPrior_LogValue=@(x,Shape0,Scale0) -Scale0./(exp(x))-Shape0*x+Shape0*log(Scale0);


Shape0=Target.PowerShape;
Scale0=Target.PowerScale;


SMCSampler.NbMCMCMoves=CurrentNbMCMCMoves;

SMCSampler.NormalizedWeights=zeros(1,SMCSampler.NbPart,SMCSampler.NbIter);
SMCSampler.LogWeights=zeros(1,SMCSampler.NbPart,SMCSampler.NbIter);
SMCSampler.LogWeightsSaved=zeros(1,SMCSampler.NbPart,SMCSampler.NbIter);
SMCSampler.LogLike=zeros(1,SMCSampler.NbPart,SMCSampler.NbIter);
SMCSampler.SavedLogLike=zeros(1,SMCSampler.NbPart,SMCSampler.NbIter);

SMCSampler.LogPriorBeta=zeros(1,SMCSampler.NbPart,SMCSampler.NbIter);
SMCSampler.AcceptedMove.Beta=0;

SMCSampler.NbTotalMCMCMoves=0;
SMCSampler.NbResampling=0;

LogRatioNormalizing=zeros(1,SMCSampler.NbIter);

% ---- Definition of the Parameters for Prior
Prior.Beta.Mean=Target.PriorMean;%Data.MidPoint;
Prior.Beta.Cov=Target.PriorCov;%Data.Length^2;
Prior.Beta.Dimension=Target.Dim;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Sampling - Generate PArticles from prior
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

t=1;
SMCSampler.NumIter=1;
SMCSampler.Temperature=1;
for j=1:SMCSampler.NbPart

 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % --- DRAW FROM THE PRIOR ---%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




    % Draw the positions From Prior
    SMCSampler.Iter(t).Particles(j).Beta=Prior.Beta.Mean+sqrtm(Prior.Beta.Cov)*randn(Prior.Beta.Dimension,1);
    % Draw the power from the prior
    SMCSampler.Iter(t).Particles(j).LogPower=log(1./gamrnd(Shape0,1/Scale0,Target.Nb,1));


    % Computation of the prior for the position (in log)
    SMCSampler.LogPriorBeta(1,j,t)=ComputationLogNormalPDF(SMCSampler.Iter(t).Particles(j).Beta,Prior.Beta.Mean,Prior.Beta.Cov);

    % Computation of the prior for the power (in log)
    SMCSampler.LogPriorLogPower(1,j,t)=sum(LogPrior_LogValue(SMCSampler.Iter(t).Particles(j).LogPower,Shape0,Scale0));

    % Computation of the likelihood (in log)
    SMCSampler.LogLike(1,j,t)=ComputationLogLikelihood(Sensors,SMCSampler.Iter(t).Particles(j).Beta,Target,exp(SMCSampler.Iter(t).Particles(j).LogPower));


    % Computation of the importance weights (in log)
    SMCSampler.SavedLogLike(1,j,t)=SMCSampler.LogLike(1,j,t);   % Just to keep it !!

    SMCSampler.LogWeights(1,j,t)=SMCSampler.LogLike(1,j,t);




    

end

% Normalization of the importance weights
SMCSampler.NormalizedWeights(1,:,t)=exp(SMCSampler.LogWeights(1,:,t)-max(SMCSampler.LogWeights(1,:,t)));
SMCSampler.NormalizedWeights(1,:,t)=SMCSampler.NormalizedWeights(1,:,t)/sum(SMCSampler.NormalizedWeights(1,:,t));


% Computation of the ratio of the normalizing constants
LogRatioNormalizing(1,t)=log(sum(exp(SMCSampler.LogWeights(1,:,t))));

% Save the Current Particle locations
SMCSampler.Iter(t).SavedParticles=SMCSampler.Iter(t).Particles; % To be used for computing Estimation based on all the particles (Recycling)


%Compute the Effective Sample Size
Neff=1/sum(abs(SMCSampler.NormalizedWeights(1,:,t)).^2);
SMCSampler.EffectiveSampleSize(t)=Neff;

% Compute the estimation of the mean and covariance of the posterior distribution
Weights=repmat(SMCSampler.NormalizedWeights(1,:,t),Prior.Beta.Dimension,1);
WeightsPower=repmat(SMCSampler.NormalizedWeights(1,:,t),Target.Nb,1);        
BetaParticlesValues=[SMCSampler.Iter(t).Particles(:).Beta];

LogPowerParticlesValues=[SMCSampler.Iter(t).Particles(:).LogPower];

SMCSampler.Estimate.Mean=sum(Weights.*BetaParticlesValues,2);

BetaParticlesCentered=BetaParticlesValues-repmat(SMCSampler.Estimate.Mean,1,SMCSampler.NbPart);
SMCSampler.Estimate.Cov=Weights.*BetaParticlesCentered*BetaParticlesCentered';

SMCSampler.Estimate.MeanLogPower=sum(WeightsPower.*LogPowerParticlesValues,2);

LogPowerParticlesCentered=LogPowerParticlesValues-repmat(SMCSampler.Estimate.MeanLogPower,1,SMCSampler.NbPart);
SMCSampler.Estimate.CovLogPower=WeightsPower.*LogPowerParticlesCentered*LogPowerParticlesCentered';        
        

% Compute the estimate of the Evidence (Marginal Likelihood)
LogBayesEvidence=sum(LogRatioNormalizing,2)-log(SMCSampler.NbPart);
SMCSampler.LogBayesEvidence=LogBayesEvidence;





