%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     This code performs the relabeling algorithm
%
%           Inputs:
%               - ParticlesBeta: Particles for source position
%               - ParticlesLogPower: Particles for source power
%               - Weights: Normalized importance weights
%               - Display: 1 Display the output
%           Outputs:
%               - RelabeledParticlesBeta: Relabeled Particles for source position
%               - RelabeledParticlesLogPower: Relabeled Particles for source power
%               - Weights: Normalized importance weights
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Matlab Codes associated to the paper
%       ``A Bayesian Perspective on Multiple Source
%        Localization in Wireless Sensor Network'' 
% by T.L.T Nguyen, F. Septier, H. Rajaona, G.W. Peters, I. Nevat,
%       Y. Delignon
%  
%       Date: Sep. 2015
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [RelabeledParticlesBeta,RelabeledParticlesLogPower,Weights]=OnlineRelabelingv1(ParticlesBeta,ParticlesLogPower,Weights,Display)

% Definition of the cost function
LogCostFunction=@(y,x,cov) -0.5*log(det(cov))-0.5*(y-x)'*inv(cov)*(y-x);


% Thresholding weights
Threshold=1e-6;
Weights=Weights.*(Weights>Threshold);
Weights=Weights/sum(Weights);

% Sorting by Decreasing Weights ti be sure to not have zero weights at
% first steps
[C,IndSorted]=sort(Weights,'descend');
Weights=Weights(IndSorted);
ParticlesBeta=ParticlesBeta(:,IndSorted);
ParticlesLogPower=ParticlesLogPower(:,IndSorted);

NbParticlesToRelabel=length(Weights);
SizeLogPower=size(ParticlesLogPower,1);
SizeBeta=size(ParticlesBeta,1);

RelabeledParticlesBeta=zeros(size(ParticlesBeta));
RelabeledParticlesLogPower=zeros(size(ParticlesLogPower));

% Initialization
CovarianceBeta=0.01*eye(SizeBeta);
CovarianceLogPower=0.01*eye(SizeLogPower);


MeanBeta=ParticlesBeta(:,1);
RelabeledParticlesBeta(:,1)=ParticlesBeta(:,1);
MeanLogPower=ParticlesLogPower(:,1);
RelabeledParticlesLogPower(:,1)=ParticlesLogPower(:,1);

PreviousMeanBeta=MeanBeta;
PreviousMeanLogPower=MeanLogPower;
PreviousCovBeta=CovarianceBeta;
PreviousCovLogPower=CovarianceLogPower;

AllPermutations=perms([1:SizeLogPower]);
NbPermutations=size(AllPermutations,1);

%NbParticlesToRelabel





for k=2:NbParticlesToRelabel
    
    CurrentNormalizedWeights=Weights(1:k)/sum(Weights(1:k));
    
    if Weights(k)~=0
        for n=1:NbPermutations

            IndexTargetPos=reshape(repmat((AllPermutations(n,:)-1)*2,2,1)+repmat([1 2]',1,SizeBeta/2),1,SizeBeta);

            PermutedBeta=ParticlesBeta(IndexTargetPos,k);
            PermutedLogPower=ParticlesLogPower(AllPermutations(n,:),k);

            CurrentMeanBeta=PreviousMeanBeta;
            CurrentMeanLogPower=PreviousMeanLogPower;



            ParticleCenteredBeta=[RelabeledParticlesBeta(:,1:k-1) PermutedBeta]-repmat(CurrentMeanBeta,1,k);
            ParticleCenteredLogPower=[RelabeledParticlesLogPower(:,1:k-1) PermutedLogPower]-repmat(CurrentMeanLogPower,1,k);

            if k<20
                CurrentCovBeta=diag(diag(PreviousCovBeta));
                CurrentCovLogPower=diag(diag(PreviousCovLogPower));
            else
                CurrentCovBeta=((PreviousCovBeta));
                CurrentCovLogPower=((PreviousCovLogPower));
            end


            CostFunction(n)=LogCostFunction(PermutedBeta,CurrentMeanBeta,CurrentCovBeta)...
                +LogCostFunction(PermutedLogPower,CurrentMeanLogPower,CurrentCovLogPower);

        end
        [C,I]=max(CostFunction);
    else  % If Weights == 0 No Matter it won't count in the final estimation
        I=1;
    end

    
    IndexTargetPos=reshape(repmat((AllPermutations(I(1),:)-1)*2,2,1)+repmat([1 2]',1,SizeBeta/2),1,SizeBeta);
    RelabeledParticlesBeta(:,k)=ParticlesBeta(IndexTargetPos,k);
    RelabeledParticlesLogPower(:,k)=ParticlesLogPower(AllPermutations(I(1),:),k);
    
    
    
    PreviousMeanBeta=[RelabeledParticlesBeta(:,1:k)]*CurrentNormalizedWeights';
    PreviousMeanLogPower=[RelabeledParticlesLogPower(:,1:k)]*CurrentNormalizedWeights';  
    
    ParticleCenteredBeta=[RelabeledParticlesBeta(:,1:k)]-repmat(PreviousMeanBeta,1,k);
    ParticleCenteredLogPower=[RelabeledParticlesLogPower(:,1:k)]-repmat(PreviousMeanLogPower,1,k);

end


if Display
    PreviousMeanBeta
    PreviousCovBeta
    PreviousCovLogPower
    
    figure(20);clf;
    figure(21);clf;
    figure(22);clf;
    figure(23);clf;
    
    color=['k.';'r.';'g.';'kx']
    for k=1:size(MeanLogPower,1)
        CurrentIndexTarget=(k-1)*2+1:2*k;
        figure(20);
        hold on;plot(ParticlesBeta(CurrentIndexTarget(1),:),ParticlesBeta(CurrentIndexTarget(2),:),color(k,:))

        figure(21);
        hold on;plot(RelabeledParticlesBeta(CurrentIndexTarget(1),:),RelabeledParticlesBeta(CurrentIndexTarget(2),:),color(k,:))


         figure(22);
        hold on;plot(ParticlesLogPower(k,:),color(k,:))

        figure(23);
         hold on;plot(RelabeledParticlesLogPower(k,:),color(k,:))
    end
end      
        
    


